### Author: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    """
    Top interrupt

    Displays the name of the most frequent interrupt
    """
    def __init__(self):
        self.name = 'most frequent'
        self.vars = ('interrupt',)
        self.type = 's'
        self.width = 20
        self.scale = 0
        self.intset1 = [ ]
        self.open('/proc/stat')
        self.names = self.names()

    def names(self):
        ret = {}
        for line in dopen('/proc/interrupts'):
            l = line.split()
            if len(l) <= cpunr: continue
            l1 = l[0].split(':')[0]
            ### Cleanup possible names from /proc/interrupts
            l2 = ' '.join(l[cpunr+3:])
            l2 = l2.replace('_hcd:', '/')
            l2 = re.sub('@pci[:\d+\.]+', '', l2)
            l2 = re.sub('ahci\[[:\da-z\.]+\]', 'ahci', l2)
            ret[l1] = l2
        return ret

    def extract(self):
        self.output = ''
        self.val['total'] = 0.0
        for line in self.splitlines():
            if line[0] == 'intr':
                self.intset2 = [ int(i) for i in line[3:] ]

        if not self.intset1:
            self.intset1 = [ 0 for i in self.intset2 ]

        for i in range(len(self.intset2)):
            total = (self.intset2[i] - self.intset1[i]) * 1.0 / elapsed

            ### Put the highest value in self.val
            if total > self.val['total']:
                if str(i+1) in self.names:
                    self.val['name'] = self.names[str(i+1)]
                else:
                    self.val['name'] = 'int ' + str(i+1)
                self.val['total'] = total

        if step == op.delay:
            self.intset1 = self.intset2

        if self.val['total'] != 0.0:
            self.output = '%-15s%s' % (self.val['name'], cprint(self.val['total'], 'd', 5, 1000))

    def showcsv(self):
        return '%s / %f' % (self.val['name'], self.val['total'])

# vim:ts=4:sw=4:et



### Author: Adam Michel <elfurbe@furbism.com>
### Based on work by: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'nfs4 server'
        # this vars/nick pair is the ones I considered relevant. Any set of the full list would work.
        self.vars = ('read','write','readdir','getattr','setattr','commit','getfh','putfh',
                'savefh','restorefh','open','open_conf','close','access','lookup','remove')
        self.nick = ('read', 'writ', 'rdir', 'gatr','satr','cmmt','gfh','pfh','sfh','rfh',
                'open','opnc','clse','accs','lkup','rem')
        # this is every possible variable for NFSv4 server if you're into that
        #self.vars4 = ('op0-unused', 'op1-unused', 'op2-future' , 'access',
        #        'close', 'commit', 'create', 'delegpurge', 'delegreturn', 'getattr', 'getfh',
        #        'link', 'lock', 'lockt', 'locku', 'lookup', 'lookup_root', 'nverify', 'open',
        #        'openattr', 'open_conf', 'open_dgrd','putfh', 'putpubfh', 'putrootfh',
        #        'read', 'readdir', 'readlink', 'remove', 'rename','renew', 'restorefh',
        #        'savefh', 'secinfo', 'setattr', 'setcltid', 'setcltidconf', 'verify', 'write',
        #        'rellockowner')
        # I separated the NFSv41 ops cause you know, completeness.
        #self.vars41 = ('bc_ctl', 'bind_conn', 'exchange_id', 'create_ses',
        #        'destroy_ses', 'free_stateid', 'getdirdeleg', 'getdevinfo', 'getdevlist',
        #        'layoutcommit', 'layoutget', 'layoutreturn', 'secinfononam', 'sequence',
        #        'set_ssv', 'test_stateid', 'want_deleg', 'destroy_clid', 'reclaim_comp')
        # Just catin' the tuples together to make the full list.
        #self.vars = self.vars4 + self.vars41
        # these are terrible shortnames for every possible variable
        #self.nick4 = ('unsd','unsd','unsd','accs','clse','comm','crt','delp','delr','gatr','gfh',
        #        'link','lock','lckt','lcku','lkup','lkpr','nver','open','opna','opnc','opnd',
        #        'pfh','ppfh','prfh','read','rdir','rlnk','rmv','ren','rnw','rfh','sfh','snfo',
        #        'satr','scid','scic','ver','wrt','rlko')
        #self.nick41 = ('bctl','bcon','eid','cses','dses','fsid',
        #        'gdd','gdi','gdl','lcmt','lget','lrtn','sinn','seq','sets','tsts','wdel','dcid',
        #        'rcmp')
        #self.nick = self.nick4 + self.nick41
        self.type = 'd'
        self.width = 5
        self.scale = 1000
        self.open("/proc/net/rpc/nfsd")

    def check(self):
        # other NFS modules had this, so I left it. It seems to work.
        info(1, 'Module %s is still experimental.' % self.filename)

    def extract(self):
        # list of fields from /proc/net/rpc/nfsd, in order of output 
        # taken from include/linux/nfs4.h in kernel source
        nfsd4_names = ('label', 'fieldcount', 'op0-unused', 'op1-unused', 'op2-future' , 'access',
                'close', 'commit', 'create', 'delegpurge', 'delegreturn', 'getattr', 'getfh',
                'link', 'lock', 'lockt', 'locku', 'lookup', 'lookup_root', 'nverify', 'open',
                'openattr', 'open_conf', 'open_dgrd','putfh', 'putpubfh', 'putrootfh',
                'read', 'readdir', 'readlink', 'remove', 'rename','renew', 'restorefh',
                'savefh', 'secinfo', 'setattr', 'setcltid', 'setcltidconf', 'verify', 'write',
                'rellockowner', 'bc_ctl', 'bind_conn', 'exchange_id', 'create_ses',
                'destroy_ses', 'free_stateid', 'getdirdeleg', 'getdevinfo', 'getdevlist',
                'layoutcommit', 'layoutget', 'layoutreturn', 'secinfononam', 'sequence',
                'set_ssv', 'test_stateid', 'want_deleg', 'destroy_clid', 'reclaim_comp'
                )

        for line in self.splitlines():
            fields = line.split()

            if fields[0] == "proc4ops": # just grab NFSv4 stats
                assert int(fields[1]) == len(fields[2:]), ("reported field count (%d) does not match actual field count (%d)" % (int(fields[1]), len(fields[2:])))
                for var in self.vars:
                    self.set2[var] = fields[nfsd4_names.index(var)]

        for name in self.vars:
            self.val[name] = (int(self.set2[name]) - int(self.set1[name])) * 1.0 / elapsed

        if step == op.delay:
            self.set1.update(self.set2)

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'utmp'
        self.nick = ('ses', 'usr', 'adm' )
        self.vars = ('sessions', 'users', 'root')
        self.type = 'd'
        self.width = 3
        self.scale = 10

    def check(self): 
        try:
            global utmp
            import utmp
        except:
            raise Exception('Needs python-utmp module')

    def extract(self):
        for name in self.vars: self.val[name] = 0
        for u in utmp.UtmpRecord():
#           print('# type:%s pid:%s line:%s id:%s user:%s host:%s session:%s' % (i.ut_type, i.ut_pid, i.ut_line, i.ut_id, i.ut_user, i.ut_host, i.ut_session))
            if u.ut_type == utmp.USER_PROCESS:
                self.val['users'] = self.val['users'] + 1
                if u.ut_user == 'root':
                    self.val['root'] = self.val['root'] + 1
            self.val['sessions'] = self.val['sessions'] + 1

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag$wieers,com>

class dstat_plugin(dstat):
    """
    Provide Dstat's number of voluntary and involuntary context switches.

    This plugin provides a unique view of the number of voluntary and
    involuntary context switches of the Dstat process itself. It may help
    to vizualise the performance of Dstat and its selection of plugins.
    """
    def __init__(self):
        self.name = 'contxt sw'
        self.vars = ('voluntary', 'involuntary', 'total')
        self.type = 'd'
        self.width = 3
        self.scale = 100

    def extract(self):
        res = resource.getrusage(resource.RUSAGE_SELF)

        self.set2['voluntary'] = float(res.ru_nvcsw)
        self.set2['involuntary'] = float(res.ru_nivcsw)
        self.set2['total'] = (float(res.ru_nvcsw) + float(res.ru_nivcsw))

        for name in self.vars:
            self.val[name] = (self.set2[name] - self.set1[name]) * 1.0 / elapsed

        if step == op.delay:
            self.set1.update(self.set2)

# vim:ts=4:sw=4:et



### Authority: Jason Friedland <thesuperjason@gmail.com>

# This plugin has been tested with:
# - Dstat 0.6.7
# - CentOS release 5.4 (Final)
# - Python 2.4.3
# - Squid 2.6 and 2.7
 
global squidclient_options
squidclient_options = os.getenv('DSTAT_SQUID_OPTS') # -p 8080
 
class dstat_plugin(dstat):
    '''
    Provides various Squid statistics.
    '''
    def __init__(self):
        self.name = 'squid status'
        self.type = 's'
        self.width = 5
        self.scale = 1000
        self.vars = ('Number of file desc currently in use',
            'CPU Usage, 5 minute avg',
            'Total accounted',
            'Number of clients accessing cache',
            'Mean Object Size')
        self.nick = ('fdesc',
            'cpu5',
            'mem',
            'clnts',
            'objsz')

    def check(self):
        if not os.access('/usr/sbin/squidclient', os.X_OK):
            raise Exception('Needs squidclient binary')
        cmd_test('/usr/sbin/squidclient %s mgr:info' % squidclient_options)
        return True
 
    def extract(self):
        try:
            for l in cmd_splitlines('/usr/sbin/squidclient %s mgr:info' % squidclient_options, ':'):
                if l[0].strip() in self.vars:
                    self.val[l[0].strip()] = l[1].strip()
                    break
        except IOError as e:
            if op.debug > 1: print('%s: lost pipe to squidclient, %s' % (self.filename, e))
            for name in self.vars: self.val[name] = -1
        except Exception as e:
            if op.debug > 1: print('%s: exception' (self.filename, e))
            for name in self.vars: self.val[name] = -1

# vim:ts=4:sw=4:et



### Authority: dag@wieers.com

### For more information, see:
###     http://eaglet.rain.com/rick/linux/schedstat/

class dstat_plugin(dstat):
    """
    Name and total amount of CPU time consumed in milliseconds of the process
    that has the highest total amount of cputime for the measured timeframe.

    On a system with one CPU and one core, the total cputime is 1000ms. On a
    system with two cores the total cputime is 2000ms.
    """

    def __init__(self):
        self.name = 'highest total'
        self.vars = ('cputime process',)
        self.type = 's'
        self.width = 17
        self.scale = 0
        self.pidset1 = {}

    def check(self):
        if not os.access('/proc/self/schedstat', os.R_OK):
            raise Exception('Kernel has no scheduler statistics [CONFIG_SCHEDSTATS], use at least 2.6.12')

    def extract(self):
        self.output = ''
        self.pidset2 = {}
        self.val['result'] = 0
        for pid in proc_pidlist():
            try:
                ### Reset values
                if pid not in self.pidset1:
                    self.pidset1[pid] = {'run_ticks': 0}

                ### Extract name
                name = proc_splitline('/proc/%s/stat' % pid)[1][1:-1]

                ### Extract counters
                l = proc_splitline('/proc/%s/schedstat' % pid)
            except IOError:
                continue
            except IndexError:
                continue

            if len(l) != 3: continue

            self.pidset2[pid] = {'run_ticks': int(l[0])}

            totrun = (self.pidset2[pid]['run_ticks'] - self.pidset1[pid]['run_ticks']) * 1.0 / elapsed

            ### Get the process that spends the most jiffies
            if totrun > self.val['result']:
                self.val['result'] = totrun
                self.val['pid'] = pid
                self.val['name'] = getnamebypid(pid, name)

        if step == op.delay:
            self.pidset1 = self.pidset2

        if self.val['result'] != 0.0:
            self.output = '%-*s%s' % (self.width-4, self.val['name'][0:self.width-4], cprint(self.val['result'], 'd', 4, 100))

        ### Debug (show PID)
#       self.output = '%*s %-*s' % (5, self.val['pid'], self.width-6, self.val['name'])

    def showcsv(self):
        return '%s / %.4f' % (self.val['name'], self.val['result'])

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag$wieers,com>

class dstat_plugin(dstat):
    """
    Provide memory information related to the dstat process.

    The various values provide information about the memory usage of the
    dstat process. This plugin gives you the possibility to follow memory
    usage changes of dstat over time. It may help to vizualise the
    performance of Dstat and its selection of plugins.
    """
    def __init__(self):
        self.name = 'dstat memory usage'
        self.vars = ('virtual', 'resident', 'shared', 'data')
        self.type = 'd'
        self.open('/proc/%s/statm' % ownpid)

    def extract(self):
        l = self.splitline()
#        l = linecache.getline('/proc/%s/schedstat' % self.pid, 1).split()
        self.val['virtual'] = int(l[0]) * pagesize / 1024
        self.val['resident'] = int(l[1]) * pagesize / 1024
        self.val['shared'] = int(l[2]) * pagesize / 1024
#        self.val['text'] = int(l[3]) * pagesize / 1024
#        self.val['library'] = int(l[4]) * pagesize / 1024
        self.val['data'] = int(l[5]) * pagesize / 1024

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    """
    Power usage information from ACPI.

    Displays the power usage in watt per hour of your system's battery using
    ACPI information. This information is only available when the battery is
    being used (or being charged).
    """

    def __init__(self):
        self.name = 'power'
        self.nick = ( 'usage', )
        self.vars = ( 'rate', )
        self.type = 'f'
        self.width = 5
        self.scale = 1
        self.rate = 0
        self.batteries = []
        for battery in os.listdir('/proc/acpi/battery/'):
            for line in dopen('/proc/acpi/battery/'+battery+'/state').readlines():
                l = line.split()
                if len(l) < 2: continue
                self.batteries.append(battery)
                break

    def check(self):
        if not self.batteries:
            raise Exception('No battery information found, no power usage statistics')

    def extract(self):
        amperes_drawn = 0
        voltage = 0
        watts_drawn = 0
        for battery in self.batteries:
            for line in dopen('/proc/acpi/battery/'+battery+'/state').readlines():
                l = line.split()
                if len(l) < 3: continue
                if l[0] == 'present:' and l[1] != 'yes': continue
                if l[0:2] == ['charging','state:'] and l[2] != 'discharging':
                    voltage = 0
                    break
                if l[0:2] == ['present','voltage:']:
                    voltage = int(l[2]) / 1000.0
                elif l[0:2] == ['present','rate:'] and l[3] == 'mW':
                    watts_drawn = int(l[2]) / 1000.0
                elif l[0:2] == ['present','rate:'] and l[3] == 'mA':
                    amperes_drawn = int(l[2]) / 1000.0

            self.rate = self.rate + watts_drawn + voltage * amperes_drawn

        ### Return error if we found no information
        if self.rate == 0:
            self.rate = -1

        if op.update:
            self.val['rate'] = self.rate / elapsed
        else:
            self.val['rate'] = self.rate

        if step == op.delay:
            self.rate = 0

# vim:ts=4:sw=4:et



### Author: dag@wieers.com

class dstat_plugin(dstat):
    """
    CPU frequency in percentage as reported by ACPI.
    """

    def __init__(self):
        self.name = 'frequency'
        self.type = 'p'
        self.width = 4
        self.scale = 34

    def check(self): 
        for cpu in glob.glob('/sys/devices/system/cpu/cpu[0-9]*'):
            if not os.access(cpu+'/cpufreq/scaling_cur_freq', os.R_OK):
                raise Exception('Cannot access acpi %s frequency information' % os.path.basename(cpu))

    def vars(self):
        ret = []
        for name in glob.glob('/sys/devices/system/cpu/cpu[0-9]*'):
            ret.append(os.path.basename(name))
        ret.sort()
        return ret
#       return os.listdir('/sys/devices/system/cpu/')

    def nick(self):
        return [name.lower() for name in self.vars]

    def extract(self):
        for cpu in self.vars:
            for line in dopen('/sys/devices/system/cpu/'+cpu+'/cpufreq/scaling_max_freq').readlines():
                l = line.split()
                max = int(l[0])
            for line in dopen('/sys/devices/system/cpu/'+cpu+'/cpufreq/scaling_cur_freq').readlines():
                l = line.split()
                cur = int(l[0])
            ### Need to close because of bug in sysfs (?)
            dclose('/sys/devices/system/cpu/'+cpu+'/cpufreq/scaling_cur_freq')
            self.set1[cpu] = self.set1[cpu] + cur * 100.0 / max

            if op.update:
                self.val[cpu] = self.set1[cpu] / elapsed
            else:
                self.val[cpu] = self.set1[cpu]

            if step == op.delay:
                self.set1[cpu] = 0

# vim:ts=4:sw=4:et



### Author: Dean Wilson <dean.wilson@gmail.com>
global socket
import socket

class dstat_plugin(dstat):
    """
    Memcache hit count plugin.

    Displays the number of memcache get_hits and get_misses.
    """
    def __init__(self):
        self.name = 'Memcache'
        self.nick = ('Hit', 'Miss')
        self.vars = ('get_hits', 'get_misses')
        self.type = 'd'
        self.width = 4
        self.scale = 999

    def check(self):
        return 1

    def extract(self):
        stats = self.get_stats()

        # Store the data we're looking for
        for key in self.vars:
            self.set2[key] = int(stats.get(key, -1))

        # If we're updating do the math to count how many happened in this interval
        if update:
            for key in self.vars:
                self.val[key] = (self.set2[key] - self.set1[key]) / elapsed

        if step == op.delay:
            self.set1.update(self.set2)

    def get_stats(self):
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except socket.error as err:
            print("socket creation failed with error %s" % (err))
            sys.exit(2)

        # Check some ENV variables first, then fall back to the defaults
        port = os.environ.get('DOOL_MEMCACHE_PORT', 11211)
        host = os.environ.get('DOOL_MEMCACHE_HOST', "127.0.0.1")

        try:
            s.connect((host, port))
        except:
            print("Memcache: Error connecting to %s:%d" % (host, port))
            sys.exit(1)

        # Send the stats command and then the quit command right after
        s.sendall(b"stats\n")
        s.sendall(b"quit\n")

        # Read everything in to a buffer
        buf = ""
        while 1:
            data = s.recv(1024)
            if not data:
                break
            buf += data.decode("utf-8")

        lines = buf.split("\r\n")
        stats = {}

        for line in lines:
            parts = line.split(" ")
            if parts[0] == "STAT":
                stats[parts[1]] = parts[2]

        return stats



class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'snooze'
        self.vars = ('snooze',)
        self.type = 's'
        self.width = 6
        self.scale = 0
        self.before = time.time()

    def extract(self):
        now = time.time()
        if loop != 0:
            self.val['snooze'] = now - self.before
        else:
            self.val['snooze'] = self.before
        if step == op.delay:
            self.before = now

    def show(self):
        if self.val['snooze'] > step + 1:
            return ansi['default'] + '     -'

        if op.blackonwhite:
            textcolor = 'black'
            if step != op.delay:
                textcolor = 'darkgray'
        else:
            textcolor = 'white'
            if step != op.delay:
                textcolor = 'gray'

        snoze, c = fchg(self.val['snooze'], 6, 1000)

        return color[textcolor] + snoze



global mysql_options
mysql_options = os.getenv('DSTAT_MYSQL')

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'mysql io'
        self.nick = ('recv', 'sent')
        self.vars = ('Bytes_received', 'Bytes_sent')

    def check(self): 
        if not os.access('/usr/bin/mysql', os.X_OK):
            raise Exception('Needs MySQL binary')
        try:
            self.stdin, self.stdout, self.stderr = dpopen('/usr/bin/mysql -n %s' % mysql_options)
        except IOError:
            raise Exception('Cannot interface with MySQL binary')

    def extract(self):
        try:
            self.stdin.write("show status like 'Bytes_%';\n")
            for line in readpipe(self.stdout):
                l = line.split()
                if len(l) < 2: continue
                if l[0] in self.vars:
                    self.set2[l[0]] = float(l[1])

            for name in self.vars:
                self.val[name] = (self.set2[name] - self.set1[name]) * 1.0 / elapsed

            if step == op.delay:
                self.set1.update(self.set2)

        except IOError as e:
            if op.debug > 1: print('%s: lost pipe to mysql, %s' % (self.filename, e))
            for name in self.vars: self.val[name] = -1

        except Exception as e:
            if op.debug > 1: print('dstat_innodb_buffer: exception', e)
            for name in self.vars: self.val[name] = -1

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag@wieers.com>

class dstat_plugin(dstat):
    def __init__(self):
        self.name = 'postfix'
        self.nick = ('inco', 'actv', 'dfrd', 'bnce', 'defr')
        self.vars = ('incoming', 'active', 'deferred', 'bounce', 'defer')
        self.type = 'd'
        self.width = 4
        self.scale = 100

    def check(self):
        if not os.access('/var/spool/postfix/active', os.R_OK):
            raise Exception('Cannot access postfix queues')

    def extract(self):
        for item in self.vars:
            self.val[item] = len(glob.glob('/var/spool/postfix/'+item+'/*/*'))

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag$wieers,com>
### Author: Sven-Hendrik Haase <sh@lutzhaase.com>

class dstat_plugin(dstat):
    """
    Percentage of remaining battery power as reported by ACPI.
    """
    def __init__(self):
        self.name = 'battery'
        self.type = 'p'
        self.width = 4
        self.scale = 34
        self.battery_type = "none"

    def check(self):
        if os.path.exists('/proc/acpi/battery/'):
            self.battery_type = "procfs"
        elif glob.glob('/sys/class/power_supply/BAT*'):
            self.battery_type = "sysfs"
        else:
            raise Exception('No ACPI battery information found.')

    def vars(self):
        ret = []
        if self.battery_type == "procfs":
            for battery in os.listdir('/proc/acpi/battery/'):
                for line in dopen('/proc/acpi/battery/'+battery+'/state').readlines():
                    l = line.split()
                    if len(l) < 2: continue
                    if l[0] == 'present:' and l[1] == 'yes':
                        ret.append(battery)
        elif self.battery_type == "sysfs":
            for battery in glob.glob('/sys/class/power_supply/BAT*'):
                for line in dopen(battery+'/present').readlines():
                    if int(line[0]) == 1:
                        ret.append(os.path.basename(battery))
        ret.sort()
        return ret

    def nick(self):
        return [name.lower() for name in self.vars]

    def extract(self):
        for battery in self.vars:
            if self.battery_type == "procfs":
                for line in dopen('/proc/acpi/battery/'+battery+'/info').readlines():
                    l = line.split()
                    if len(l) < 4: continue
                    if l[0] == 'last':
                        full = int(l[3])
                        break
                for line in dopen('/proc/acpi/battery/'+battery+'/state').readlines():
                    l = line.split()
                    if len(l) < 3: continue
                    if l[0] == 'remaining':
                        current = int(l[2])
                        break
                if current:
                    self.val[battery] = current * 100.0 / full
                else:
                    self.val[battery] = -1
            elif self.battery_type == "sysfs":
                for line in dopen('/sys/class/power_supply/'+battery+'/capacity').readlines():
                    current = int(line)
                    break
                if current:
                    self.val[battery] = current
                else:
                    self.val[battery] = -1

# vim:ts=4:sw=4:et



### Author: Dag Wieers <dag$wieers,com>

class dstat_plugin(dstat):
    """
    Provide CPU information related to the dstat process.

    This plugin shows the CPU utilization for the dstat process itself,
    including the user-space and system-space (kernel) utilization and
    a total of both. On a system with one cpu and one core, the total
    cputime is 1000ms. On a system with 2 cores the total is 2000ms.
    It may help to vizualise the performance of Dstat and its selection
    of plugins.
    """
    def __init__(self):
        self.name = 'dstat cpu'
        self.vars = ('user', 'system', 'total')
        self.nick = ('usr', 'sys', 'tot')
        self.type = 'p'
        self.width = 3
        self.scale = 100

    def extract(self):
        res = resource.getrusage(resource.RUSAGE_SELF)

        self.set2['user'] = float(res.ru_utime)
        self.set2['system'] = float(res.ru_stime)
        self.set2['total'] = float(res.ru_utime) + float(res.ru_stime)

        for name in self.vars:
            self.val[name] = (self.set2[name] - self.set1[name]) * 100.0 / elapsed / cpunr

        if step == op.delay:
            self.set1.update(self.set2)

# vim:ts=4:sw=4:et



### Author: <gianfranco@mongodb.com>

global mongodb_user
mongodb_user = os.getenv('DSTAT_MONGODB_USER') or os.getenv('USER')

global mongodb_pwd
mongodb_pwd = os.getenv('DSTAT_MONGODB_PWD')

global mongodb_host
mongodb_host = os.getenv('DSTAT_MONGODB_HOST') or '127.0.0.1:27017'

class dstat_plugin(dstat):
  """
  Plugin for MongoDB.
  """
  def __init__(self):
    global pymongo
    import pymongo

    try:
      self.m = pymongo.MongoClient(mongodb_host)
      if mongodb_pwd:
        self.m.admin.authenticate(mongodb_user, mongodb_pwd)
      self.db = self.m.admin
    except Exception as e:
      raise Exception('Cannot interface with MongoDB server: %s' % e)

    stats  = self.db.command("listDatabases")
    self.dbList = []
    for db in stats.get('databases'):
      self.dbList.append(db.get('name'))

    line = self.db.command("serverStatus")
    if 'storageEngine' in line:
      self.storageEngine = line.get('storageEngine').get('name')
    else:
      self.storageEngine = 'mmapv1'

    self.name    = 'mongodb stats'
    self.nick    = ('dsize', 'isize', 'ssize')
    self.vars    = ('dataSize', 'indexSize', 'storageSize')
    self.type    = 'b'
    self.width   = 5
    self.scale   = 2
    self.count   = 1

    if self.storageEngine == 'mmapv1':
      self.nick  = self.nick + ('fsize',)
      self.vars  = self.vars + ('fileSize',)


  def extract(self):
    self.set = {}

    # refresh the database list every 10 iterations
    if (self.count % 10) == 0:
      stats  = self.m.admin.command("listDatabases")
      self.dbList = []
      for db in stats.get('databases'):
        self.dbList.append(db.get('name'))
    self.count += 1

    for name in self.vars:
      self.set[name] = 0

    for db in self.dbList:
      self.db = self.m.get_database(db)
      stats = self.db.command("dbStats")
      for name in self.vars:
        self.set[name] += int(stats.get(name)) / (1024 * 1024)
    self.val = self.set



class dstat_plugin(dstat):
    """
    ZFS on Linux ZIL (ZFS Intent Log)

    Data is extracted from /proc/spl/kstat/zfs/zil
    """
    def __init__(self):
        self.name = 'ZFS ZIL'
        self.nick = ('count', 'bytes')
        self.vars = ('zil_itx_metaslab_slog_count', 'zil_itx_metaslab_slog_bytes')
        self.types = ('d', 'b')
        self.scales = (1000, 1024)
        self.counter = (True, True)
        self.open('/proc/spl/kstat/zfs/zil')

    def extract(self):
        for l in self.splitlines():
            if len(l) < 2: continue
            l[0].split()
            name = l[0]
            if name in self.vars:
                self.set2[name] = int(l[2])

        for i, name in enumerate (self.vars):
            if self.counter[i]:
                self.val[name] = (self.set2[name] - self.set1[name]) * 1.0 / elapsed
            else:
                self.val[name] = self.set2[name]

        if step == op.delay:
            self.set1.update(self.set2)

# vim:ts=4:sw=4:et
